var documenterSearchIndex = {"docs":
[{"location":"#GSVD.jl","page":"GSVD.jl","title":"GSVD.jl","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"GSVD.jl is a Julia program to compute the following  generalized singular value decomposition (GSVD) of an m-by-n matrix A and a p-by-n matrix B defined in LAPACK [1].","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\nA = UCRQ^T quad B = VSRQ^T\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where U, V and Q are m-by-m, p-by-p and n-by-n orthogonal matrices, respectively. R = left  beginarraycc          0  R_1         endarray right is r-by-n and R_1 is r-by-r is upper triangular and nonsingular, where r = textrank(A B) leq n. C and S are m-by-r and p-by-r  real, non-negative and diagonal matrices, respectively,  and C^T C + S^T S = I_r. Specifically, C and S have the following structures: ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"if m geq r:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"<img src='figs/case1.png'>","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"if m  r:  ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"<img src='figs/case2.png'>","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where Sigma_1 and Sigma_2 are nonnegative diagonal matrices,  Sigma_2 is nonsingular and ell = textrank(B). ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"gsvd(A, B) returns an object F such that","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"F.U is the m-by-m orthogonal matrix U,\nF.V is the p-by-p orthogonal matrix V,\nF.Q is the n-by-n orthogonal matrix Q,\nF.R1 is the r-by-r nonsingular upper      triangular matrix R_1 in the r-by-n matrix R,\nF.alpha and F.beta are arrays of length r to      store the diagonal elements of C and S, respectively. \nF.k and F.l are integers such that     r = F.k + F.l is the rank of A B.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"The diagonals of C and S can be stored in arrays F.alpha and F.beta of length r as follows.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"if m geq r:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\n    tt Falpha1  =  cdots = tt Falphar-l = 1 \n\ttt Falphar-l+i = (Sigma_1)_ii \n\ttextfor i = 1 cdots ell   \n\ttt Fbeta1  = cdots = tt Fbetar-l = 0  \n\ttt Fbetar-l+i = (Sigma_2)_ii \n\ttextfor i = 1 cdots ell\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"if m  r:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\n    tt Falpha1  = cdots = tt Falphar-l = 1 \n\ttt Falphar-l+i = (Sigma_1)_ii  \n\ttextfor i = 1 cdots m+ell-r \n\ttt Falpham+1 = cdots = tt Falphar = 0 \n \ttt Fbeta1  = cdots = tt Fbetar-l = 0 \n\ttt Fbetar-l+i = (Sigma_2)_ii  textfor i = 1 cdots m+ell-r \n\ttt Fbetam+1 = cdots = tt Fbetar = 1\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"In addition,  F.C, F.S, and F.R  returns the matrices C, S and R explicitly. ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"The ratios F.vals[i] equiv F.alpha[i]/F.beta[i] for i = 1 ldots r are called the generalized singular values (gsv) of (A B).","category":"page"},{"location":"#Example","page":"GSVD.jl","title":"Example","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Here's an example of how to use GSVD.jl.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"julia> using GSVD\n\njulia> A = [1. 2 3 0; 5 4 2 1; 0 3 5 2; 2 1 3 3; 2 0 5 3];\njulia> B = [1. 0 3 -1; -2 5 0 1; 4 2 -1 2];\n\njulia> F = gsvd(A, B);\n\njulia> F.C\n5×4 SparseArrays.SparseMatrixCSC{Float64,Int64} with 4 stored entries:\n  [1, 1]  =  1.0\n  [2, 2]  =  0.894685\n  [3, 3]  =  0.600408\n  [4, 4]  =  0.27751\n\njulia> F.S\n3×4 SparseArrays.SparseMatrixCSC{Float64,Int64} with 3 stored entries:\n  [1, 2]  =  0.446698\n  [2, 3]  =  0.799694\n  [3, 4]  =  0.960723\n\njulia> F.R\n4×4 Array{Float64,2}:\n 5.74065  -7.07986   0.125979  -0.316232\n 0.0      -7.96103  -2.11852   -2.98601 \n 0.0       0.0       5.72211   -0.43623 \n 0.0       0.0       0.0        5.66474\n \njulia> [A; B] ≈ [F.U*F.C; F.V*F.S]*F.R*F.Q'\ntrue","category":"page"},{"location":"#Algorithm","page":"GSVD.jl","title":"Algorithm","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"GSVD.jl is an implementation of a four-step algorithm using LAPACK routine DGGSVD3 for pre-processing and  a specialized 2-by-1 CS decomposition due to Van Loan [2]. ","category":"page"},{"location":"#GSVD-module","page":"GSVD.jl","title":"GSVD module","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"GSVD.jl is in the GSVD module, which consists of","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"|── src\n|     |── GSVD.jl\n|     └── preproc.jl\n|     └── householderqr.jl\n|     └── csd.jl\n|── test\n|     |── ...","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"The main function gsvd(A, B)  calls preproc, householderqr, csd2by1,  and then post-process for implementing the four-step  algorithm.","category":"page"},{"location":"#Installation","page":"GSVD.jl","title":"Installation","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"To install GSVD, from the Julia REPL, type ] to enter the Pkg REPL mode and run:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"pkg> add GSVD","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"or using the Pkg API:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"julia> import Pkg; Pkg.add(\"GSVD\")","category":"page"},{"location":"#Performance","page":"GSVD.jl","title":"Performance","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"gsvd(A,B) exhibits the same numerical stability  as svd(A, B), a wrapper to LAPACK routine DGGSVD3  available in Julia 1.3 on a large set of random matrices of  different dimensions.   The following plots show that for large matrices, gsvd(A, B) is  up to 15times faster than svd(A, B). However, for matrices of dimensions small than 40, gsvd(A, B)  is slightly slower. ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"<table border=\"0\"><tr>\n<td>\n\t<figure>\n\t\t<img src='figs/small_m_p_n_5_4_3.png'><br>\n\t</figure>\n</td>\n<td>\n\t<figure>\n\t\t<img src='figs/m_p_n_5_4_3.png'><br>\n\t</figure>\n</td>\n</tr></table>\n\n<br/>\n<table border=\"0\"><tr>\n<td>\n\t<figure>\n\t\t<img src='figs/small_m_p_n_5_3_4.png'><br>\n\t</figure>\n</td>\n<td>\n\t<figure>\n\t\t<img src='figs/m_p_n_5_3_4.png'><br>\n\t</figure>\n</td>\n</tr></table>","category":"page"},{"location":"#Properties-of-GSVD","page":"GSVD.jl","title":"Properties of GSVD","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"When B is square and nonsingular,  the GSVD of (AB) is equivalent to the SVD of AB^-1:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\n    A B^-1 = U (CS^-1) V^T\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"If we rewrite the GSVD as","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"AleftbeginarrayccQ_1  Q_2endarrayright = UCleftbeginarraycc0  R_1endarray right quad BleftbeginarrayccQ_1  Q_2endarrayright = UCleftbeginarraycc0  R_1endarray right where Q_1 is n-by-(n-r) and Q_2 is n-by-r, then null(A)cap null(B) = span{Q_1} i.e., Q_1 is an orthonormal basis of the common nullspace of A and B.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"By the GSVD of (A B), the matrices A^TA and B^TB  are simultaenously diagonalized under the congruence transformation:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\n    X^TA^TAX = beginbmatrix\n            0  0 \n            0  C^TC\n             endbmatrix  quad\n    X^TB^TBX = beginbmatrix\n                0  0 \n                0  S^TS\n             endbmatrix\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\nX = Q beginbmatrix\n\tI  \n  \t  R_1^-1\nendbmatrix\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Consequently, the non-trivial eigenpairs of (A^TA B^TB) are given by ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\nA^TAX_i+n-r = lambda_i B^TBX_i+n-r\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where for i = 1 cdots r, lambda_i = sigma^2_i are the generalized singular values of (A B) and  X_i+n-r is the (i+n-r)-th column of X.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"The aforementioned properties are presented  in most definitions of the generalized singular value decomposition  in literature, except MATLAB.","category":"page"},{"location":"#GSVD.jl-VS.-gsvd.m","page":"GSVD.jl","title":"GSVD.jl VS. gsvd.m","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"In MATLAB [3], the GSVD of an m-by-n matrix A and  a p-by-n matrix B is defined by","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\nA = UCX^T quad B = VSX^T\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where U and V are m-by-m and p-by-p orthogonal matrices,  respectively.  X is n-by-q, where q = minm + p n. C and S are m-by-q and p-by-q nonnegative diagonal  matrices and C^T C + S^T S = I. The nonzero elements of S are always on its main diagonal. The nonzero elements of C are on the diagonal  textdiag(C max0q-m). If m geq q, this is the main diagonal of C.  If m  q, the upper right m-by-m block of C is diagonal [4]. ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Let C^T C = diag(alpha_1^2 cdots alpha_q^2), and  S^T S = textdiag(beta_1^2 cdots beta_q^2) for i = 1 ldots q. The ratios alpha_ibeta_i are the generalized singular values of  (A B).","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"As shown by the following example,  the GSVD defined in MATLAB: ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"does not reveal the rank of AB, \ndoes not simultaneously diagonalize the matrices A^TA and B^TB under the congruence transformation. Therefore it may not have the relation with the non-trivial eigenpairs of (A^TA B^T B). ","category":"page"},{"location":"#Example-of-GSVD.jl-vs.-gsvd.m","page":"GSVD.jl","title":"Example of GSVD.jl vs. gsvd.m","text":"","category":"section"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Consider ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"    beginaligned\n        A = beginbmatrix\n            0  -frac38  0  fracsqrt38\n            -frac38  0  fracsqrt38  0\n            0  -frac3sqrt38  0  frac38\n            fracsqrt38  0  -frac18  0\n        endbmatrix quad\n        B = beginbmatrix\n            0  fracsqrt38  0  frac78\n            -frac3sqrt38  0  frac38  0\n            0  -frac58  0  -fracsqrt38\n            frac38  0  -fracsqrt38  0\n        endbmatrix\n    endaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"where textrank(A) = 2, textrank(B) = 3 and textrank(AB) = 3. The exact GSVD of (AB) as defined in LAPACK is given by","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\nU = V = Q = beginbmatrix\n            frac12  0  fracsqrt32  0 \n            0   -fracsqrt32  0  frac12 \n            fracsqrt32   0   -frac12  0 \n            0  frac12  0  fracsqrt32 \n      endbmatrix  quad \nR_1 =  I_3 quad\nC = beginbmatrix\n\tfracsqrt32  0  0 \n\t0  frac12  0 \n\t0  0  0 \n\t0  0  0 \n\tendbmatrix  \n\tquad \nS = beginbmatrix\n\tfrac12  0  0 \n\t0  fracsqrt32  0 \n\t0  0  1 \n\t0  0  0 \n\tendbmatrix\nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Therefore, the generalized singular values are sigma_1 = sqrt3,  sigma_2 = frac1sqrt3,  sigma_3 = frac01 = 0.  Furthermore, the non-trivial eigenvalues of (A^TA B^T B) are lambda_1 = 3 lambda_2 = frac13 lambda_3 = 0.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"By GSVD.jl, we have","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"julia> F.U = \n4×4 Array{Float64,2}: \n 0.5        0.0      -0.866025   0.0     \n 0.0       -0.866025  0.0        0.5     \n 0.866025   0.0       0.5        0.0     \n 0.0        0.5       0.0        0.866025\njulia> F.V\n4×4 Array{Float64,2}:\n 0.5        0.0       -0.866025  0.0     \n 0.0       -0.866025   0.0       0.5     \n 0.866025   0.0        0.5       0.0     \n 0.0        0.5        0.0       0.866025\njulia> F.Q\n4×4 Array{Float64,2}:\n 0.5        0.0        0.866025   0.0     \n 0.0       -0.866025   0.0       -0.5     \n 0.866025   0.0       -0.5        0.0     \n 0.0        0.5        0.0       -0.866025\njulia> F.R\n3×4 Array{Float64,2}:\n  1.0          0.0  2.22045e-16\n  0.0          1.0  0.0        \n -1.11022e-16  0.0  1.0\njulia> F.alpha \n3-element Array{Float64,1}:\n 0.8660254037844384\n 0.5               \n 0.0 \njulia> F.beta   \n3-element Array{Float64,1}:\n 0.5               \n 0.8660254037844384\n 1.0000000000000002\njulia> F.k\n0\njulia> F.l\n3 ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Consequently, the numerical rank of AB is  F.k + F.l = 3 and the generalized singular values  of (A B) are:","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"beginaligned\ntt Falpha1tt Fbeta1  = 17321 approx sigma_1 = sqrt3 \ntt Falpha2tt Fbeta2  = 05774 approx sigma_2 = frac1sqrt3 \ntt Falpha3tt Fbeta3  = 00  = sigma_3 = 0  \nendaligned","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"and agree with the exact generalized singular values up to machine precision.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"In contrast, by MATLAB's gsvd.m, we have","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":">> U =\n    0.4899    0.0848   -0.1131    0.8602 \n    0.4123   -0.8944   -0.0750   -0.1565 \n   -0.2829   -0.0573   -0.9566    0.0409 \n    0.7141    0.4355   -0.2580   -0.4836 \n\n>> V = \n   -0.8660   -0.0110   -0.4943    0.0742\n   -0.0000   -0.8584    0.0947    0.5042\n    0.5000   -0.0190   -0.8562    0.1285 \n   -0.0000    0.5126    0.1163    0.8507 \n\n>> X = \n         0    0.8641   -0.0330   -0.0468 \n   -0.5000    0.0109    0.7898   -0.3550 \n   -0.0000   -0.4989    0.0190    0.0270 \n   -0.8660   -0.0063   -0.4560    0.2050 \n\n>> C = \n    0.0000         0         0         0 \n         0    0.4972         0         0 \n         0         0    0.8404         0 \n         0         0         0    0.9835 \n>> S =\n    1.0000         0         0         0 \n         0    0.8676         0         0 \n         0         0    0.5420         0 \n         0         0         0    0.1809 ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"Consequently, the generalized singular values computed by gsvd.m are  0 05731  15504 54352, which have no connection with  the non-trivial eigenvalues of (A^TA B^TB).  ","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"[1]: Edward Anderson, Zhaojun Bai, Christian Bischof, L Susan Blackford, James Demmel, Jack Dongarra, Jeremy Du Croz, Anne Greenbaum, Sven Hammarling, Alan McKenney, et al. LAPACK Users’ guide. SIAM, 1999.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"[2]: Charles Van Loan. Computing the CS and the generalized singular value decompositions. Numerische Mathematik, 46(4):479–491, 1985.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"[3]: MATLAB. Generalized singular value decomposition documentation. The MathWorks Inc., Natick, Massachusetts, 2019. Available at https://www.mathworks.com/help/matlab/ref/ gsvd.html.","category":"page"},{"location":"","page":"GSVD.jl","title":"GSVD.jl","text":"[4]: This is not in MATLAB's description of the GSVD.","category":"page"}]
}
